/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/* This XPCOM service knows how to ask Amazon.com where to find art for 
 * music albums. 
 */

if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

const STRING_BUNDLE = "chrome://albumartmanager/locale/albumartmanager.properties";
const PREF_BRANCH = "songbird.albumartmanager.fetcher.";

Cu.import('resource://app/jsmodules/sbProperties.jsm');
Cu.import("resource://app/jsmodules/StringUtils.jsm");
Cu.import("resource://app/jsmodules/ArrayConverter.jsm");
Cu.import('resource://gre/modules/XPCOMUtils.jsm');

/**
 * Since we can't use the FUEL components until after all other components have
 * been loaded we define a lazy getter here for when we need it.
 */
__defineGetter__("Application", function() {
  delete this.Application;
  return this.Application = Cc["@mozilla.org/fuel/application;1"]
                              .getService(Ci.fuelIApplication);
});


function sbAmazonCoverFetcher() {
  this._ioService = Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);

  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                             .getService(Ci.nsIStringBundleService);
  this._bundle = stringBundleService.createBundle(STRING_BUNDLE);

  this.DEBUG = Application.prefs.getValue("songbird.albumartmanager.debug", false);
	if (this.DEBUG) {
    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);
  }

	this.__defineGetter__("shortName",
	function name() {
	  return "amazon";
	});

	this.__defineGetter__("name",
	function name() {
	  return SBString(PREF_BRANCH + this.shortName + ".name",
										null,
										this._bundle);
	});

	this.__defineGetter__("description",
	function description() {
	  return SBString(PREF_BRANCH + this.shortName + ".description",
										null,
										this._bundle);
	});

	this.__defineGetter__("priority",
	function isEnabled() {
		return Application.prefs
											.getValue(PREF_BRANCH + this.shortName + ".priority",
																false);
	});
	this.__defineSetter__("priority",
	function isEnabled(aNewVal) {
		return Application.prefs
											.setValue(PREF_BRANCH + this.shortName + ".priority",
																aNewVal);
	});

	this.__defineGetter__("isEnabled",
	function isEnabled() {
		return Application.prefs
											.getValue(PREF_BRANCH + this.shortName + ".isEnabled",
																false);
	});
	this.__defineSetter__("isEnabled",
	function isEnabled(aNewVal) {
		return Application.prefs
											.setValue(PREF_BRANCH + this.shortName + ".isEnabled",
																aNewVal);
	});

}
sbAmazonCoverFetcher.prototype = {
	// XPCOM information
  className: 'sbAmazonCoverFetcher',
  classDescription: 'Amazon Cover Fetcher',
  // Please generate a new classID with uuidgen.
  classID: Components.ID('{0e027cfa-db40-4448-ad25-3ae0f1000e27}'),
  contractID: '@songbirdnest.com/Songbird/cover/amazon-fetcher;1',
  _xpcom_categories: [{
    category: "sbCoverFetcher"
  }],

  // Variables
  DEBUG: false,                       // Debug flag for _debug function
	_bundle: null,
	_shutdown: false,
	
  /**
   * Internal debugging functions
   */
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        the className pre-appended.
   * \param message String to print out.
   */
  _debug: function (message) {
    if (!this.DEBUG) return;
    try {
      dump(this.className + ": " + message + "\n");
      this._consoleService.logStringMessage(this.className + ": " + message);
    } catch (err) {
      // We do not want to throw an exception here
    }
  },
  
  /**
   * \brief Dumps out an error message with the className + ": [ERROR]"
   *        pre-appended, and will report the error so it will appear in the
   *        error console.
   * \param message String to print out.
   */
  _logError: function (message) {
    try {
      dump(this.className + ": [ERROR] - " + message + "\n");
      Cu.reportError(this.className + ": [ERROR] - " + message);
    } catch (err) {
      // We do not want to thow an exception here
    }
  },

	/**
	 * \brief Searches Amazon.com for covers for the album/artist
	 * \param aAlbumName Album name to use in search
	 * \param aArtistName Artist name to use in search
	 * \param aListener Listener to call back to on fail/success
	 * \param aItemList List of items this search pertains to
	 */
	_findCoverURL: function (aAlbumName, aArtistName, aListener, aItemList) {
		if (this._shutdown) {
			return;
		}

	  // build the amazon query
	  var amazonKey = '068VR1FECHFBJ1NBK5R2';
	  var amazonTag = 'bengtonlin-20';

		var amazonTld = Application.prefs.getValue("songbird.albumartmanager.country",
																							 "com");

	  var amazonURL = "http://webservices.amazon." + amazonTld + 
	    "/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazonKey + 
	    "&AssociateTag=" + amazonTag + "&Operation=ItemSearch&SearchIndex=Music";

	  if (aArtistName && aArtistName != "") {
	    amazonURL += '&Keywords=' + encodeURI(aArtistName);
	  }

	  if (aAlbumName && aAlbumName != "") {
	    amazonURL += "&Title=" + encodeURI(aAlbumName);
	    amazonURL += "&ResponseGroup=Small,Images";
	  } else {
	    amazonURL += "&ResponseGroup=Small,Images,Tracks";
	  }

	  var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
	    				  .createInstance(Ci.nsIXMLHttpRequest);
	  xhr.open("get", amazonURL, true);
	  xhr.onreadystatechange = function() {
	    if (xhr.readyState == 4) {
				if (this._shutdown) {
					return;
				}

				// Check for successfull results
	      try {
					if ( (xhr.status != 200) ||
							 (!Components.isSuccessCode(xhr.channel.status)) ||
							 (!xhr.responseXML) ) {
	          aListener.coverFetchFailed(aItemList);
						return;		
				  }
	      } catch (e) {
					// mozilla bug xhr.status throws error!
	        aListener.coverFetchFailed(aItemList);
	        xhr.cancel();
	        return;
	      }

	      // Find all the items in the response
	      var elements = xhr.responseXML.getElementsByTagName('Item');
	      // what order do we look for the sizes?
	      var sizes = ['Large', 'Medium', 'Small'];
	      var foundCovers = [];
	      // for each element, in relevance order...
	      for (var i=0; i<elements.length; i++) {
	        var element = elements[i];
	        // look for the size of image we want
	        for (var j=0; j<sizes.length; j++) {
						var imageURLElements = element.getElementsByTagName(sizes[j]+'Image');
						for (var k=0; k<imageURLElements.length; k++) {
							var imageURLElement = imageURLElements[k];
		          var url = imageURLElement.getElementsByTagName('URL')[0]
																		   .firstChild.nodeValue;
 							if ( (url && url != "") &&
 							     (foundCovers.indexOf(url) == -1) ) {
								// Only add unique images.
								foundCovers.push(url);
							}
						}
	        }
	      }

				// Give each item the list of covers
				for (var l=0; l<aItemList.length;l++) {
					aItemList[l] = { guid: aItemList[l], covers: foundCovers };
				}

        aListener.coverFetchSucceeded(aItemList);
	    }
	  }

	  this._debug("Sending url : " + amazonURL);
	  xhr.send(null);
	},

  /*********************************
   * sbICoverFetcher
   ********************************/
	fetchCover: function (aLibrary, aPropertyArray, aListener, aWindow) {
		if (this._shutdown) {
			return;
		}
		
		var albumName = "";
		var artistName = "";
		
		// We can only use the Album and Artist names for searching for covers
		for (var pIndex = 0; pIndex < aPropertyArray.length; pIndex++) {
			var aProperty = aPropertyArray.getPropertyAt(pIndex);
			switch (aProperty.id) {
				case SBProperties.albumName:
					albumName = aProperty.value;
				break;
				
				case SBProperties.artistName:
					artistName = aProperty.value;
				break;
			}
		}

		// Find a list of items for this search
    var self = this;
    var mediaListListener = {
			itemGuidList: [],
      onEnumerationBegin: function() {
				if (this._shutdown) {
					return Ci.sbIMediaListEnumerationListener.CANCEL;
				}
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumeratedItem: function(list, item) {
				if (this._shutdown) {
					return Ci.sbIMediaListEnumerationListener.CANCEL;
				}
        this.itemGuidList.push(item.guid);
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumerationEnd: function() {
      }
    };
    aLibrary.enumerateItemsByProperties(aPropertyArray,
                                        mediaListListener,
                                        Ci.sbIMediaList
                                          .ENUMERATIONTYPE_SNAPSHOT);		

		if (this._shutdown) {
			return;
		}
		// Now search for the covers
		this._findCoverURL(albumName,
											 artistName,
											 aListener,
											 mediaListListener.itemGuidList);
	},
	
	shutdown: function () {
	  this._shutdown = true;
	},

  /*********************************
   * nsISupports
   ********************************/
  QueryInterface: XPCOMUtils.generateQI([Ci.sbICoverFetcher])

}

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbAmazonCoverFetcher]);
}
