/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

// Constants for convience
if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

// Imports to help with some common tasks
Cu.import("resource://app/jsmodules/sbProperties.jsm");
Cu.import("resource://app/jsmodules/sbLibraryUtils.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

/**
 * Since we can't use the FUEL components until after all other components have
 * been loaded we define a lazy getter here for when we need it.
 */
__defineGetter__("Application", function() {
  delete this.Application;
  return this.Application = Cc["@mozilla.org/fuel/application;1"]
                              .getService(Ci.fuelIApplication);
});

// Constants for preferences
const PREF_COVERDOWNLOADER_DEBUG = "songbird.albumartmanager.debug";
const PREF_SAVELOCATION = "songbird.albumartmanager.cover.saveTo";
const PREF_COVERFORMAT = "songbird.albumartmanager.cover.format";

/**
 * \brief CoverDownloader downloads cover art using Mozillas
 *        nsIWebBrowserPersist and then returns the downloaded file to the
 *        sbICoverListener.
 */
function sbCoverDownloader() {
  // Check if we need debugging stuff
  this.DEBUG = Application.prefs.getValue(PREF_COVERDOWNLOADER_DEBUG, false);
  if (this.DEBUG) {
    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);
  }
  this._debug("Initialized.");
}
sbCoverDownloader.prototype = {
  className: "sbCoverDownloader",
  constructor: sbCoverDownloader,       // Constructor to this object
  classDescription: "Songbird Album Cover Downloader",
  classID: Components.ID("{1a5f4f10-5578-464b-abfd-6c1bf7973ea1}"),
  contractID: "@songbirdnest.com/Songbird/sbCoverDownloader;1",
  
  _consoleService: null,      // We use this in the debug calls

  // Global variables for when we are downloading.  
  _outputFile     : null,     // nsILocalFile we are saving to.
  _mediaItem      : null,     // Media Item we are downloding for
  _scope          : null,     // Scope of search
  _useTempLocation: false     // Use temp location when needing to save to the
                              // profile folder
}

/**
 * Internal debugging functions
 */
/**
 * \brief Dumps out a message with this.className pre-appended if the DEBUG flag
 *        is enabled.
 * \param message String to print out.
 */
sbCoverDownloader.prototype._debug =
function sbCoverDownloader__debug(message)
{
  if (!this.DEBUG) return;
  if (!this.className) return;
  if (!this._consoleService) return;
  if (!message) return;
  dump(this.className + ": " + message + "\n");
  this._consoleService.logStringMessage(this.className + ": " + message);
}

/**
 * \brief Dumps out an error message with this.className + ": [ERROR]"
 *        pre-appended, and will report the error so it will appear in the
 *        error console.
 * \param message String to print out.
 */
sbCoverDownloader.prototype._logError =
function sbCoverDownloader__logError(message)
{
  if (!this.className) return;
  if (!message) return;
  dump(this.className + ": [ERROR] - " + message + "\n");
  Cu.reportError(this.className + ": [ERROR] - " + message);
}

/* nsISupports */
sbCoverDownloader.prototype.QueryInterface =
XPCOMUtils.generateQI([Ci.nsIWebProgressListener, // For downloading images
                       Ci.sbICoverDownloader]);

/* nsIWebProgressListener methods */
// No need to implement anything in these functions
sbCoverDownloader.prototype.onLocationChange = function (a, b, c) { }
sbCoverDownloader.prototype.onProgressChange = function (a, b, c, d, e, f) { }
sbCoverDownloader.prototype.onSecurityChange = function (a, b, c) { }
sbCoverDownloader.prototype.onStatusChange = function (a, b, c, d) { }

/**
 * \brief onStateChange is fired when the state of the download changes,
 *        we are only looking for the STATE_STOP state that indicates the
 *        download is done.
 */
sbCoverDownloader.prototype.onStateChange =
function sbCoverDownloader_onStateChange(aWebProgress,
                                         aRequest,
                                         aStateFlags, 
                                         aStatus) {
  this._debug('onStateChange aStatus = ' + aStatus);

  // if we're shutting down, do nothing
  if (this._shutdown) {
    // Should we maybe stop the download?
    return;
  }

  // when the transfer is complete...
  if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
    if (aStatus == 0) {
      this._downloadFinished();
    } else {
      this._debug("Download failed for : " + this._outputFile.path);
      this._listener.coverDownloadFailed(this._mediaItem, this._scope);
    }
  }
}

sbCoverDownloader.prototype._getImageData =
function sbCoverDownloader__getImageData(imageFilePath) {
  var imageFile = Cc["@mozilla.org/file/local;1"]
                    .createInstance(Ci.nsILocalFile);
  imageFile.initWithPath(imageFilePath);
  
  var newMimeType = Cc["@mozilla.org/mime;1"]
                      .getService(Ci.nsIMIMEService)
                      .getTypeFromFile(imageFile);
  var inputStream = Cc["@mozilla.org/network/file-input-stream;1"]
                      .createInstance(Ci.nsIFileInputStream);
  inputStream.init(imageFile, 0x01, 0600, 0);
  var stream = Cc["@mozilla.org/binaryinputstream;1"]
                 .createInstance(Ci.nsIBinaryInputStream);
  stream.setInputStream(inputStream);
  var size = inputStream.available();
  // use a binary input stream to grab the bytes.
  var bis = Cc["@mozilla.org/binaryinputstream;1"].
            createInstance(Ci.nsIBinaryInputStream);
  bis.setInputStream(inputStream);
  var newImageData= bis.readByteArray(size);

  return [newMimeType, newImageData];
}
/**
 * \brief DownloadFinished completes the download operation and notifies the
 *        listener that it has completed.
 */
sbCoverDownloader.prototype._downloadFinished =
function sbCoverDownloader__downloadFinished() {
  var ioService = Cc["@mozilla.org/network/io-service;1"]
                    .getService(Ci.nsIIOService);

  // If we are using a temporary file then we have to copy it to the profile
  // folder.
  if (this._useTempLocation) {
    var imageMimeType;
    var imageData;
    [imageMimeType, imageData] = this._getImageData(this._outputFile.path);
    
    var metadataImageScannerService =
                     Cc["@songbirdnest.com/Songbird/Metadata/ImageScanner;1"]
                       .getService(Ci.sbIMetadataImageScanner);
  
    var newFile = metadataImageScannerService.saveImageDataToFile(
                                                              imageData,
                                                              imageData.length,
                                                              imageMimeType);
    var fph = ioService.getProtocolHandler("file")
                       .QueryInterface(Ci.nsIFileProtocolHandler);
    this._outputFile = fph.getFileFromURLSpec(newFile);
  }

  this._debug("Saved image file to : " + this._outputFile.path);

  try {
    var outFile = ioService.newFileURI(this._outputFile).spec;
    this._listener.coverDownladSucceeded(this._mediaItem,
                                         this._scope,
                                         outFile);
  } catch (err) {
    this._logError("Unable to get newFileURI for: " +
                   this._outputFile.path + "\n" + "Error = " + err);
    this._listener.coverDownloadFailed(this._mediaItem, this._scope);
  }
}

/**
 * \brief MakeFileNameSafe takes a file name and changes any invalid characters
 *        into valid ones.
 *        TODO: Check if the ioService will do this for use with newFileURI
 *        NOTE: We can not use the platform since each platform can use many
 *              file systems, so we just account for all :P
 * \param aFileName - file name to make safe
 * \return valid version of aFileName.
 */
sbCoverDownloader.prototype._makeFileNameSafe =
function sbCoverDownloader__makeFileNameSafe(aFileName) {
  var newFileName;
  // Windows file systems have so many bad characters
  // * " / \ : | ? < >
  newFileName = aFileName.replace(/[*|\"|\/|\\|\||:|\?|<|>|]/g, "_");
  // OS X file systems only have : but also can not start with .
  if (newFileName.indexOf(".") == 0) {
    newFileName = newFileName.slice(1);
  }
  return newFileName;
}

/**
 * \brief Helper function for getting a property from a media item with a
 *        default value if none found.
 * \param aMediaItem item to get property from
 * \param aPropertyName property to get from item
 * \param aDefaultValue default value to use if property had no value
 * \return property value or default value passed in if not available
 */
sbCoverDownloader.prototype._getProperty =
function sbCoverDownloader__getProperty(aMediaItem,
                                        aPropertyName,
                                        aDefaultValue) {
  var propertyValue = aMediaItem.getProperty(aPropertyName);
  if ( (propertyValue == null) ||
       (propertyValue == "") ) {
    return aDefaultValue;
  } else {
    return propertyValue;
  }
}

/**
 * \brief Takes the Cover Format preference and appends the information on to
 *        an nsIFile.
 * \param aFile the nsIFile to append the information on to.
 * \param aMediaItem item to use for generating file name.
 * \param aExtension optional extension to add on to the filename.
 * \return the completed nsIFile
 */
sbCoverDownloader.prototype._convertCoverFormat =
function sbCoverDownloader__convertCoverFormat(aFile,
                                               aMediaItem,
                                               aExtension)
{
  var albumName = this._getProperty(aMediaItem, SBProperties.albumName, "");
  var albumArtist = this._getProperty(aMediaItem, SBProperties.artistName, "");
  var trackName = this._getProperty(aMediaItem, SBProperties.trackName, "");

  var coverFormat = Application.prefs.getValue(PREF_COVERFORMAT,
                                               "%artist% - %album%");
  
  coverFormat = coverFormat.replace(/%album%/gi,
                                    this._makeFileNameSafe(albumName));
  coverFormat = coverFormat.replace(/%artist%/gi,
                                    this._makeFileNameSafe(albumArtist));
  coverFormat = coverFormat.replace(/%track%/gi,
                                    this._makeFileNameSafe(trackName));

  if (aExtension) {
    coverFormat = coverFormat + aExtension;
  }

  // Have to append each folder if the user is using \ or / in the format
  // we can not seem to append something like "folder/folder/folder" :P
  try {
    if ( (coverFormat.indexOf("/") > 0) ||
         (coverFormat.indexOf("\\") > 0) ) {
      coverFormat = coverFormat.replace("\\", "/");
      var sections = coverFormat.split("/");
      
      for (var i = 0; i < sections.length; i++) {
        var appendFolder = this._makeFileNameSafe(sections[i]);
        aFile.append(appendFolder);
      }
    } else {
      coverFormat = this._makeFileNameSafe(coverFormat);
      aFile.append(coverFormat);
    }
  } catch (err) {
    this._logError("Unable to create downloadLocation with " + coverFormat);
    aFile = null;
  }
  
  return aFile;
}

/**
 * \brief Determines the download location based on preferences.
 *
 * \param aMediaItem item to generate download url for.
 * \param aExtension optional extension to add on to the filename.
 * \return a nsIFile of the location to download to minus an extension if
 *         aExtension is null, otherwise full location.
 */
sbCoverDownloader.prototype._getCoverDownloadLocation =
function sbCoverDownloader__getCoverDownloadLocation(aMediaItem,
                                                     aExtension)
{
  var contentURL = aMediaItem.getProperty(SBProperties.contentURL);
  
  // Default to the profile folder
  var saveLocation = Application.prefs.getValue(PREF_SAVELOCATION, 0);
  var downloadLocation = null;
  this._useTempLocation = false;
  
  switch(saveLocation) {
    default:
    case 0: // Profile
      this._debug("Using profile location.");
      // Here we need to create a tempFile to save to which we will move
      // to the profile folder when we are done downloading because we need
      // the image data to generate the proper file name.
      downloadLocation = Cc["@mozilla.org/file/directory_service;1"]
                           .getService(Ci.nsIProperties)
                           .get("TmpD", Ci.nsIFile);

      var uuidGenerator = Cc["@mozilla.org/uuid-generator;1"]
                            .getService(Ci.nsIUUIDGenerator);
      var uuid = uuidGenerator.generateUUID();
      var uuidFileName = uuid.toString();
      
      if (aExtension) { uuidFileName = uuidFileName + aExtension; }
      downloadLocation.append(uuidFileName);
      downloadLocation.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0777);

      var registerFileForDelete = Cc["@mozilla.org/uriloader/external-helper-app-service;1"]
                                   .getService(Ci.nsPIExternalAppLauncher);
      registerFileForDelete.deleteTemporaryFileOnExit(downloadLocation);
      
      this._useTempLocation = true;
    break;
  
    case 1: // Album
      this._debug("Using item location.");
      // Here we save to where the album is
      var uri;
      try {
        var ioService =  Cc['@mozilla.org/network/io-service;1']
                           .getService(Ci.nsIIOService);
        var fph = ioService.getProtocolHandler("file")
                           .QueryInterface(Ci.nsIFileProtocolHandler);
        var file = fph.getFileFromURLSpec(contentURL);
        if (file.parent) {
          downloadLocation = file.parent;
          downloadlocation = this._convertCoverFormat(downloadLocation,
                                                      aMediaItem,
                                                      aExtension);
        }
      } catch (err) {
        this._logError("Error getting content location: " + err);
      }
    break;
  
    case 2: // Custom
      this._debug("Using custom location.");
      // Here we save to a custom location
      var prefService = Cc["@mozilla.org/preferences-service;1"]
                          .getService(Ci.nsIPrefBranch);
      downloadLocation = prefService.getComplexValue(
                                       "songbird.albumartmanager.cover.folder",
                                        Ci.nsILocalFile);
      if (downloadLocation) {
        downloadLocation = this._convertCoverFormat(downloadlocation,
                                                    aMediaItem,
                                                    aExtension);
      }
    break;
  }

  return downloadLocation;
}

/* sbICoverDownloader methods */
sbCoverDownloader.prototype.downloadCover =
function sbCoverDownloader_downloadCover( aURI,
                                          aMediaItem,
                                          aListener,
                                          aScope) {
  this._debug('downloadCover: ' + aURI);
  
  // Check if we already have this item in the correct location
  var currentLocation = aMediaItem.getProperty(SBProperties.primaryImageURL);
  if (currentLocation == aURI) {
    aListener.coverDownladSucceeded(aMediaItem,
                                    aScope,
                                    aURI);
    return;
  }
  
  // we'll need the io service for a couple of things...
  var ioService =  Cc['@mozilla.org/network/io-service;1']
                    .getService(Ci.nsIIOService);

  // track shutdown state
  this._shutdown = false;

  // default to .jpg
  var extension = '.jpg';
  try {
    // try to extract the extension from download URL
    extension = aURI.match(/\.[a-zA-Z0-9]+$/)[0];
  } catch(e) {
    this._logError("Failed to extract the extension from :" + aURI + " - " + e);
  }

  // work out where we'll save this
  var file = this._getCoverDownloadLocation(aMediaItem, extension);
  
  // Check again if we are saving to the same place
  currentLocation = ioService.newFileURI(file).spec;
  if (currentLocation == aURI) {
    aListener.coverDownladSucceeded(aMediaItem,
                                    aScope,
                                    aURI);
    return;
  }
  
  if (file) {
    if (!file.exists()) {
      // Make sure we can create the file
      try {
        file.create(Ci.NORMAL_FILE_TYPE, 0664);
      } catch (err) {
        this._logError("Unable to create file: [" + file.path + "] - " + err);
        this._listener.coverDownloadFailed(aMediaItem, aScope);
        return;
      }
    }
    
    // Save this for when we have finished downloading
    this._outputFile = file;
    this._listener = aListener;
    this._mediaItem = aMediaItem;
    this._scope = aScope;
  
    this._wbp = Cc['@mozilla.org/embedding/browser/nsWebBrowserPersist;1']
      .createInstance(Ci.nsIWebBrowserPersist);
    this._wbp.persistFlags = Ci.nsIWebBrowserPersist.PERSIST_FLAGS_NONE;
    this._wbp.progressListener = this;
    this._debug('beginning download');
    this._wbp.saveURI(ioService.newURI(aURI, null, null),
                      null,
                      null,
                      null, 
                      null,
                      file);
  } else {
    this._debug("couldn't determine the cover location");
    this._listener.coverDownloadFailed(aMediaItem, aScope);
  }
}

sbCoverDownloader.prototype.shutdown =
function sbCoverDownloader_shutdown() {
  this._debug('shutdown');
  this._shutdown = true;
}

/**
 * XPCOM Implementation
 */
function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbCoverDownloader]);
}
