/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/* This XPCOM service knows how to find album art that's stored on your
 * local disk.
 */

if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

const STRING_BUNDLE = "chrome://albumartmanager/locale/albumartmanager.properties";
const PREF_BRANCH = "songbird.albumartmanager.fetcher.";

Cu.import('resource://app/jsmodules/sbProperties.jsm');
Cu.import("resource://app/jsmodules/StringUtils.jsm");
Cu.import("resource://app/jsmodules/ArrayConverter.jsm");
Cu.import('resource://gre/modules/XPCOMUtils.jsm');

/**
 * Since we can't use the FUEL components until after all other components have
 * been loaded we define a lazy getter here for when we need it.
 */
__defineGetter__("Application", function() {
  delete this.Application;
  return this.Application = Cc["@mozilla.org/fuel/application;1"]
                              .getService(Ci.fuelIApplication);
});

function sbFileCoverFetcher() {
  this.DEBUG = Application.prefs.getValue("songbird.albumartmanager.debug", false);
  if (this.DEBUG) {
    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);
  }

  this._ioService = Cc['@mozilla.org/network/io-service;1']
    .getService(Ci.nsIIOService);
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                             .getService(Ci.nsIStringBundleService);
  this._bundle = stringBundleService.createBundle(STRING_BUNDLE);

  this.__defineGetter__("shortName",
  function name() {
    return "file";
  });

  this.__defineGetter__("name",
  function name() {
    return SBString(PREF_BRANCH + this.shortName + ".name",
                    null,
                    this._bundle);
  });

  this.__defineGetter__("description",
  function description() {
    return SBString(PREF_BRANCH + this.shortName + ".description",
                    null,
                    this._bundle);
  });

	this.__defineGetter__("priority",
	function isEnabled() {
		return Application.prefs
											.getValue(PREF_BRANCH + this.shortName + ".priority",
																false);
	});
	this.__defineSetter__("priority",
	function isEnabled(aNewVal) {
		return Application.prefs
											.setValue(PREF_BRANCH + this.shortName + ".priority",
																aNewVal);
	});

  this.__defineGetter__("isEnabled",
  function isEnabled() {
    return Application.prefs
                      .getValue(PREF_BRANCH + this.shortName + ".isEnabled",
                                false);
  });
  this.__defineSetter__("isEnabled",
  function isEnabled(aNewVal) {
    return Application.prefs
                      .setValue(PREF_BRANCH + this.shortName + ".isEnabled",
                                aNewVal);
  });
}
sbFileCoverFetcher.prototype = {
  className: "sbFileCoverFetcher",
  classDescription: "Songbird Local File Album Cover Fetcher",
  // Please generate a new classID with uuidgen.
  classID: Components.ID("{ca2da771-c58a-44cf-8b58-139f553d62a1}"),
  contractID: "@songbirdnest.com/Songbird/cover/file-fetcher;1",
  _xpcom_categories: [{
    category: "sbCoverFetcher"
  }],

  // Variables
  DEBUG: false,                       // Debug flag for _debug function
  _bundle: null,
  _shutdown: false,
  
  /**
   * Internal debugging functions
   */
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        the className pre-appended.
   * \param message String to print out.
   */
  _debug: function (message) {
    if (!this.DEBUG) return;
    try {
      dump(this.className + ": " + message + "\n");
      this._consoleService.logStringMessage(this.className + ": " + message);
    } catch (err) {
      // We do not want to throw an exception here
    }
  },
  
  /**
   * \brief Dumps out an error message with the className + ": [ERROR]"
   *        pre-appended, and will report the error so it will appear in the
   *        error console.
   * \param message String to print out.
   */
  _logError: function (message) {
    try {
      dump(this.className + ": [ERROR] - " + message + "\n");
      Cu.reportError(this.className + ": [ERROR] - " + message);
    } catch (err) {
      // We do not want to thow an exception here
    }
  },

  /*********************************
   * sbICoverFetcher
   ********************************/
  fetchCover: function (aLibrary, aPropertyArray, aListener, aWindow) {
    if (this._shutdown) {
      return;
    }

    // Find a list of items for this search
    var self = this;
    var mediaListListener = {
      itemGuidList: [],
      onEnumerationBegin: function() {
        if (this._shutdown) {
          return Ci.sbIMediaListEnumerationListener.CANCEL;
        }
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumeratedItem: function(list, item) {
        if (this._shutdown) {
          return Ci.sbIMediaListEnumerationListener.CANCEL;
        }
        this.itemGuidList.push(item.guid);
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumerationEnd: function() {
      }
    };
    aLibrary.enumerateItemsByProperties(aPropertyArray,
                                        mediaListListener,
                                        Ci.sbIMediaList
                                          .ENUMERATIONTYPE_SNAPSHOT);   

    if (this._shutdown) {
      return;
    }
    var aItemList = mediaListListener.itemGuidList;

    if (aWindow) {
      // Open the file picker
      var filePicker = Cc["@mozilla.org/filepicker;1"]
                          .createInstance(Ci.nsIFilePicker);
      var windowTitle = SBString("albumartmanager.filepicker.windowtitle", null, this._bundle);
      filePicker.init( aWindow, windowTitle,
                      Ci.nsIFilePicker.modeOpen);
      var fileResult = filePicker.show();
      if (fileResult == Ci.nsIFilePicker.returnOK) {
        // Give each item the list of covers
        for (var l=0; l<aItemList.length;l++) {
          aItemList[l] = { guid: aItemList[l],
                           covers: ["file://" + filePicker.file.path]
                         };
        }
        aListener.coverFetchSucceeded(aItemList);
      } else {
        this._debug("User canceled file picker dialog");
        aListener.coverFetchFailed(aItemList);
      }
    } else {
      var itemsWithCover = [];
      var itemsWithoutCover = [];
      for (var i=0; i<aItemList.length;i++) {
        var aMediaItem = aLibrary.getMediaItem(aItemList[i]);
        var extCheck = /.+\.(jpg|jpeg|png|gif|tif|tiff|bmp])$/;
        var contentURL = aMediaItem.getProperty(SBProperties.contentURL);
        try {
          var fph = this._ioService.getProtocolHandler("file")
                        .QueryInterface(Ci.nsIFileProtocolHandler);
          var file = fph.getFileFromURLSpec(contentURL);
          if (file.parent) {
            location = file.parent;
          }
        } catch (err) {
          this._logError("Error getting content location: " + err);
          location = null;
        }

        if (location) {
          var foundCovers = [];
          var dirList = location.directoryEntries;
          while (dirList.hasMoreElements()) {
            var testFile = dirList.getNext().QueryInterface(Ci.nsIFile);
            if (testFile.isFile) {
              var result = testFile.leafName.match(extCheck);
              if (result) {
                var outFile = this._ioService.newFileURI(testFile).spec;
                this._debug("Found possible cover image as: " + outFile);
                foundCovers.push(outFile);
              } else {
                this._debug("Bummer " + testFile.leafName + " did not match");
              }
            }
          }
          
          if (foundCovers.length > 0) {
            itemsWithCover.push({ guid: aMediaItem.guid, covers: foundCovers });
          } else {
            itemsWithoutCover.push(aMediaItem.guid);
          }
        } else {
          itemsWithoutCover.push(aMediaItem.guid);
        }
      }
    
      if (itemsWithCover.length > 0) {
        aListener.coverFetchSucceeded(itemsWithCover);
      }
      if (itemsWithoutCover.length > 0) {
        aListener.coverFetchFailed(itemsWithoutCover);
      }

    }
  },

  shutdown: function () {
    this._shutdown = true;
  },

  /*********************************
   * nsISupports
   ********************************/
  QueryInterface: XPCOMUtils.generateQI([Ci.sbICoverFetcher])

}

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbFileCoverFetcher]);
}


sbFileCoverFetcher.prototype.fetchCoverForMediaItem = 
function sbFileCoverFetcher_fetchCoverForMediaItem(aMediaItem, aListener, aWindow) {
  if (this._shutdown) {
    return;
  }

  if (aWindow) {
    // Open the file picker
    var filePicker = Cc["@mozilla.org/filepicker;1"]
                        .createInstance(Ci.nsIFilePicker);
    var windowTitle = SBString("albumartmanager.filepicker.windowtitle", null, this._bundle);
    filePicker.init( aWindow, windowTitle,
                    Ci.nsIFilePicker.modeOpen);
    var fileResult = filePicker.show();
    if (fileResult == Ci.nsIFilePicker.returnOK) {
      aListener.coverFetchSucceeded(aMediaItem,
                                    null,
                                    "file://" + filePicker.file.path);
    } else {
      this._debug("User canceled file picker dialog");
      aListener.coverFetchFailed(aMediaItem, null);
    }
  } else {
    // where would we store this media item's cover?
    var location = this._albumArtService.getCoverDownloadLocation(aMediaItem);
    var extensions = ['.jpg','.jpeg', '.png', '.gif', '.tif', '.tiff', '.bmp'];
    var extCheck = /.+\.(jpg|jpeg|png|gif|tif|tiff|bmp])$/;
    var fph = this._ioService.getProtocolHandler("file")
                  .QueryInterface(Ci.nsIFileProtocolHandler);

    if (location) {
      var filePath = location.path;
      for (var i = 0; i < extensions.length; i++) {
        var fullFile = filePath + extensions[i];
        var file = fph.getFileFromURLSpec("file://" + fullFile);
        this._debug("Checking for file: " + file.path);
        if (file.exists()) {
          var outFile = this._ioService.newFileURI(file).spec;
          aListener.coverFetchSucceeded(aMediaItem, null, outFile);
          return;
        }
      }
    }

    // If we didn't find a cover that is based on the user preferences
    // Then we should scan through a bunch of defaults filenames in the
    // 
    var contentURL = aMediaItem.getProperty(SBProperties.contentURL);
    try {
      var file = fph.getFileFromURLSpec(contentURL);
      if (file.parent) {
        location = file.parent;
      }
    } catch (err) {
      this._logError("Error getting content location: " + err);
      location = null;
    }

    var dirList = location.directoryEntries;
    while (dirList.hasMoreElements()) {
      var testFile = dirList.getNext().QueryInterface(Ci.nsIFile);
      if (testFile.isFile) {
        var result = testFile.leafName.match(extCheck);
        if (result) {
          var outFile = this._ioService.newFileURI(testFile).spec;
          this._debug("Found possible cover image as: " + outFile);
          aListener.coverFetchSucceeded(aMediaItem, null, outFile);
          return;
        } else {
          this._debug("Bummer " + testFile.leafName + " did not match");
        }
      }
    }
    
    // Failed to find a cover
    this._debug("No covers found on file system.");
    aListener.coverFetchFailed(aMediaItem, null);
  }
}
