/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/


/**
 * This XPCOM service knows how to retrieve album art from the metadata of an
 * item. 
 */

if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

const STRING_BUNDLE = "chrome://albumartmanager/locale/albumartmanager.properties";
const PREF_BRANCH = "songbird.albumartmanager.fetcher.";

Cu.import('resource://app/jsmodules/sbProperties.jsm');
Cu.import("resource://app/jsmodules/StringUtils.jsm");
Cu.import("resource://app/jsmodules/ArrayConverter.jsm");
Cu.import('resource://gre/modules/XPCOMUtils.jsm');

/**
 * Since we can't use the FUEL components until after all other components have
 * been loaded we define a lazy getter here for when we need it.
 */
__defineGetter__("Application", function() {
  delete this.Application;
  return this.Application = Cc["@mozilla.org/fuel/application;1"]
                              .getService(Ci.fuelIApplication);
});

function sbMetadataCoverFetcher() {
  this.DEBUG = Application.prefs.getValue("songbird.albumartmanager.debug", false);
	if (this.DEBUG) {
    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);
  }

  this._metadataImageScanner = Cc["@songbirdnest.com/Songbird/Metadata/ImageScanner;1"]
                            .getService(Ci.sbIMetadataImageScanner);
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                             .getService(Ci.nsIStringBundleService);
  this._bundle = stringBundleService.createBundle(STRING_BUNDLE);

	this.__defineGetter__("shortName",
	function name() {
	  return "metadata";
	});

	this.__defineGetter__("name",
	function name() {
	  return SBString(PREF_BRANCH + this.shortName + ".name",
										null,
										this._bundle);
	});

	this.__defineGetter__("description",
	function description() {
	  return SBString(PREF_BRANCH + this.shortName + ".description",
										null,
										this._bundle);
	});

	this.__defineGetter__("priority",
	function isEnabled() {
		return Application.prefs
											.getValue(PREF_BRANCH + this.shortName + ".priority",
																false);
	});
	this.__defineSetter__("priority",
	function isEnabled(aNewVal) {
		return Application.prefs
											.setValue(PREF_BRANCH + this.shortName + ".priority",
																aNewVal);
	});

	this.__defineGetter__("isEnabled",
	function isEnabled() {
		return Application.prefs
											.getValue(PREF_BRANCH + this.shortName + ".isEnabled",
																false);
	});
	this.__defineSetter__("isEnabled",
	function isEnabled(aNewVal) {
		return Application.prefs
											.setValue(PREF_BRANCH + this.shortName + ".isEnabled",
																aNewVal);
	});

}
sbMetadataCoverFetcher.prototype = {
	// XPCOM information
  className: "sbMetadataCoverFetcher",
  classDescription: 'Metadata Cover Fetcher',
  // Please generate a new classID with uuidgen.
  classID: Components.ID('{58f15ee8-783c-4447-b178-1451bfe725bf}'),
  contractID: '@songbirdnest.com/Songbird/cover/metadata-fetcher;1',
  _xpcom_categories: [{
    category: "sbCoverFetcher"
  }],

  // Variables
  DEBUG: false,                       // Debug flag for _debug function
	_bundle: null,
	_metadataImageScanner: null,
	_shutdown: false,
	
  /**
   * Internal debugging functions
   */
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        the className pre-appended.
   * \param message String to print out.
   */
  _debug: function (message) {
    if (!this.DEBUG) return;
    try {
      dump(this.className + ": " + message + "\n");
      this._consoleService.logStringMessage(this.className + ": " + message);
    } catch (err) {
      // We do not want to throw an exception here
    }
  },
  
  /**
   * \brief Dumps out an error message with the className + ": [ERROR]"
   *        pre-appended, and will report the error so it will appear in the
   *        error console.
   * \param message String to print out.
   */
  _logError: function (message) {
    try {
      dump(this.className + ": [ERROR] - " + message + "\n");
      Cu.reportError(this.className + ": [ERROR] - " + message);
    } catch (err) {
      // We do not want to thow an exception here
    }
  },

  /*********************************
   * sbICoverFetcher
   ********************************/
	fetchCover: function (aLibrary, aPropertyArray, aListener, aWindow) {
	  if (this._shutdown) {
	    return;
	  }

    var self = this;
    var mediaListListener = {
			itemsWithCover: [],
			itemsWithoutCover: [],
      onEnumerationBegin: function() {
			  if (self._shutdown) {
			    return Ci.sbIMediaListEnumerationListener.CANCEL;
			  }
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumeratedItem: function(list, item) {
			  if (self._shutdown) {
			    return Ci.sbIMediaListEnumerationListener.CANCEL;
			  }
				
				var aCover = self._metadataImageScanner.fetchCoverForMediaItem(item);
				if (aCover && aCover != "") {
					this.itemsWithCover.push({ guid: item.guid, covers: [aCover]});
				} else {
					this.itemsWithoutCover.push(item.guid);
				}
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumerationEnd: function() { }
    };

    aLibrary.enumerateItemsByProperties(aPropertyArray,
                                        mediaListListener,
                                        Ci.sbIMediaList
                                          .ENUMERATIONTYPE_SNAPSHOT);
	  if (this._shutdown) {
	    return;
	  }

		if (mediaListListener.itemsWithCover.length > 0) {
			aListener.coverFetchSucceeded(mediaListListener.itemsWithCover);
		}
		if (mediaListListener.itemsWithoutCover.length > 0) {
			aListener.coverFetchFailed(mediaListListener.itemsWithoutCover);
		}
	},
	
	shutdown: function () {
	  this._shutdown = true;
	},

  /*********************************
   * nsISupports
   ********************************/
  QueryInterface: XPCOMUtils.generateQI([Ci.sbICoverFetcher])

}

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([sbMetadataCoverFetcher]);
}




